# coding: utf-8
# /*##########################################################################
#
# Copyright (c) 2016-2017 European Synchrotron Radiation Facility
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
#
# ###########################################################################*/


__authors__ = ["J. Garriga"]
__license__ = "MIT"
__date__ = "30/03/2021"

import logging
import os

from silx.gui import qt
from i3dxrd.data_selection import load_process_data
from silx.gui.dialog.DataFileDialog import DataFileDialog

_logger = logging.getLogger(__file__)


class DatasetSelectionWidget(qt.QTabWidget):
    """
    Widget that creates a dataset from a list of files or from a single filename.
    It lets the user add the first filename of a directory of files, or to
    upload manually each of the files to be read.
    If both options are filled up, only the files in the list of filenames
    are read.
    """

    sigProgressChanged = qt.Signal(int)

    def __init__(self, parent=None):
        qt.QMainWindow.__init__(self, parent)

        self._isHdf5Checkbox = qt.QCheckBox("Hdf5 file")

        # Raw data
        self._rawFilenameData = FilenameSelectionWidget(
            "Load data first file", parent=self
        )
        self._rawFilesData = FilesSelectionWidget(parent=self)
        self._inDiskCB = qt.QCheckBox("Use data from disk")
        rawData = qt.QWidget(self)
        rawData.setLayout(qt.QVBoxLayout())
        rawData.layout().addWidget(self._isHdf5Checkbox)
        rawData.layout().addWidget(self._rawFilenameData)
        rawData.layout().addWidget(self._rawFilesData)
        rawData.layout().addWidget(self._inDiskCB)
        self._inDiskCB.setChecked(True)
        self.addTab(rawData, "raw data")

        self._darkFilename = FilenameSelectionWidget(
            "Load dark file", parent=self, fileMode=qt.QFileDialog.ExistingFile
        )
        self._floodFilename = FilenameSelectionWidget(
            "Load flood file", parent=self, fileMode=qt.QFileDialog.ExistingFile
        )
        self._splineFilename = FilenameSelectionWidget(
            "Load spline file", parent=self, fileMode=qt.QFileDialog.ExistingFile
        )

        peaksData = qt.QWidget(self)
        peaksData.setLayout(qt.QVBoxLayout())
        peaksData.layout().addWidget(self._darkFilename)
        peaksData.layout().addWidget(self._floodFilename)
        peaksData.layout().addWidget(self._splineFilename)

        self.addTab(peaksData, "peak search data")

        self._inDisk = True
        self._isH5 = False

        self._dataset = None
        self.indices = None

        self.getRawFilenames = self._rawFilesData.getFiles
        self.getRawFilename = self._rawFilenameData.getFilename
        self.getDarkFilename = self._darkFilename.getFilename
        self.getFloodFilename = self._floodFilename.getFilename
        self.getSplineFilename = self._splineFilename.getFilename
        self.setRawFilenames = self._rawFilesData.setFiles
        self.setRawFilename = self._rawFilenameData.setFilename
        self.setDarkFilename = self._darkFilename.setFilename
        self.setFloodFilename = self._floodFilename.setFilename
        self.setSplineFilename = self._splineFilename.setFilename

        self._inDiskCB.stateChanged.connect(self.__inDisk)
        self._isHdf5Checkbox.stateChanged.connect(self._changeDataType)

    def _changeDataType(self, state):

        self._isH5 = bool(state)
        self._rawFilenameData.isHDF5(state)
        if state:
            self._rawFilesData.hide()
        else:
            self._rawFilesData.show()

    def updateProgress(self, progress):
        self.sigProgressChanged.emit(progress)

    def loadDataset(self):
        """
        Loads the dataset from the filenames.
        """

        dark_filename = self.getDarkFilename() if self.getDarkFilename() else None
        flood_filename = self.getFloodFilename() if self.getFloodFilename() else None
        spline_filename = self.getSplineFilename() if self.getSplineFilename() else None
        filenames = self._rawFilesData.getFiles()
        if not filenames:
            filenames = self._rawFilenameData.getFilename()
        self._dataset = load_process_data(
            filenames,
            in_memory=not self._inDisk,
            dark_filename=dark_filename,
            flood_filename=flood_filename,
            spline_filename=spline_filename,
            isH5=self._isH5,
        )
        return self.dataset is not None and self.dataset.nframes != 0

    @property
    def dataset(self):
        return self._dataset

    def __inDisk(self, inDisk):
        self._inDisk = bool(inDisk)


class FilesSelectionWidget(qt.QWidget):
    """
    Widget used to get one or more files from the computer and add them to a list.
    """

    def __init__(self, parent=None):
        qt.QWidget.__init__(self, parent)

        self._files = []

        self.setLayout(qt.QVBoxLayout())
        self._table = self._init_table()
        self._addButton = qt.QPushButton("Add")
        self._rmButton = qt.QPushButton("Remove")
        self.layout().addWidget(self._table)
        self.layout().addWidget(self._addButton)
        self.layout().addWidget(self._rmButton)
        self._addButton.clicked.connect(self._addFiles)
        self._rmButton.clicked.connect(self._removeFiles)

    def _init_table(self):

        table = qt.QTableWidget(0, 1, parent=self)
        table.horizontalHeader().hide()
        # Resize horizontal header to fill all the column size
        if hasattr(table.horizontalHeader(), "setSectionResizeMode"):  # Qt5
            table.horizontalHeader().setSectionResizeMode(0, qt.QHeaderView.Stretch)
        else:  # Qt4
            table.horizontalHeader().setResizeMode(0, qt.QHeaderView.Stretch)

        return table

    def _addFiles(self):
        """
        Opens the file dialog and let's the user choose one or more files.
        """
        dialog = qt.QFileDialog(self)
        dialog.setFileMode(qt.QFileDialog.ExistingFiles)

        if not dialog.exec_():
            dialog.close()
            return

        for file in dialog.selectedFiles():
            self.addFile(file)

    def _removeFiles(self):
        """
        Removes the selected items from the table.
        """
        selectedItems = self._table.selectedItems()
        if selectedItems is not None:
            for item in selectedItems:
                self._files.remove(item.text())
                self._table.removeRow(item.row())

    def addFile(self, file):
        """
        Adds a file to the table.

        :param str file: filepath to add to the table.
        """
        assert os.path.isfile(file)
        item = qt.QTableWidgetItem()
        item.setText(file)
        row = self._table.rowCount()
        self._table.setRowCount(row + 1)
        self._table.setItem(row, 0, item)
        self._files.append(file)

    def getFiles(self):
        return self._files

    def setFiles(self, files):
        """
        Adds a list of files to the table.

        :param array_like files: List to add
        """
        for file in files:
            self.addFile(file)

    def getDir(self):
        if len(self._files):
            return os.path.dirname(self._files[0])
        return None


class FilenameSelectionWidget(qt.QWidget):
    """
    Widget used to obtain a filename (manually or from a file)
    """

    filenameChanged = qt.Signal()

    def __init__(
        self,
        buttonText,
        parent=None,
        fileMode=qt.QFileDialog.AnyFile,
        acceptMode=qt.QFileDialog.AcceptOpen,
    ):
        qt.QWidget.__init__(self, parent)

        self._isH5 = False
        self._filename = None
        self._filenameLE = qt.QLineEdit("", parent=self)
        # self._filenameLE.editingFinished.connect(self.filenameChanged)
        self._addButton = qt.QPushButton(buttonText, parent=self)
        # self._okButton =  qt.QPushButton("Ok", parent=self)
        self._addButton.pressed.connect(self._uploadFilename)
        # self._okButton.pressed.connect(self.close)
        self._fileMode = fileMode
        self._acceptMode = acceptMode
        self.setLayout(qt.QHBoxLayout())

        self.layout().addWidget(self._filenameLE)
        self.layout().addWidget(self._addButton)
        # self.layout().addWidget(self._okButton)

    def isHDF5(self, isH5):
        self._isH5 = isH5

    def _uploadFilename(self):
        """
        Loads the file from a FileDialog.
        """
        if self._isH5:
            fileDialog = DataFileDialog()

            if fileDialog.exec_():
                self._filename = fileDialog.selectedDataUrl().path()
                self._filenameLE.setText(self._filename)
                self.filenameChanged.emit()
            else:
                _logger.warning("Could not open file")
        else:
            fileDialog = qt.QFileDialog()
            fileDialog.setFileMode(self._fileMode)
            fileDialog.setAcceptMode(self._acceptMode)

            if fileDialog.exec_():
                self._filenameLE.setText(fileDialog.selectedFiles()[0])
                self.filenameChanged.emit()
            else:
                _logger.warning("Could not open file")

    def getFilename(self):
        return str(self._filenameLE.text())

    def setFilename(self, filename):
        self._filename = filename
        self._filenameLE.setText(str(filename))



from typing import Iterable, Optional, Union
from i3dxrd.dataset import Dataset


def load_process_data(
    filenames: Union[str, Iterable[str]] = None,
    nframes: Optional[int] = None,
    in_memory: bool = True,
    dark_filename: Optional[str] = None,
    flood_filename: Optional[str] = None,
    spline_filename: Optional[str] = None,
    isH5: bool = False,
):
    """When `filenames` is a string, it will be treated as a file pattern."""

    if isinstance(filenames, str):
        dataset = Dataset(
            first_filename=filenames,
            in_memory=in_memory,
            dark_filename=dark_filename,
            flood_filename=flood_filename,
            spline_filename=spline_filename,
            isH5=isH5,
        )
    else:
        filenames = list(filenames)
        dataset = Dataset(
            filenames=filenames,
            in_memory=in_memory,
            dark_filename=dark_filename,
            flood_filename=flood_filename,
            spline_filename=spline_filename,
            isH5=isH5,
        )

    return dataset



# coding: utf-8
# /*##########################################################################
#
# Copyright (c) 2016-2017 European Synchrotron Radiation Facility
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
#
# ###########################################################################*/


__authors__ = ["J. Garriga"]
__license__ = "MIT"
__date__ = "28/01/2021"

import logging
import numpy

from silx.gui import qt
from silx.gui.plot import PlotWindow
from silx.utils.enum import Enum as _Enum
from ImageD11.refinegrains import refinegrains
from .dataSelectionWidget import FilenameSelectionWidget

_logger = logging.getLogger(__file__)


class Lattice(_Enum):
    triclinic = "triclinic"
    monoclinic_a = "monoclinic_a"
    monoclinic_b = "monoclinic_b"
    monoclinic_c = "monoclinic_c"
    orthorhombic = "orthorhombic"
    tetragonal = "tetragonal"
    trigonalP = "trigonalP"
    trigonalH = "trigonalH"
    cubic = "cubic"


class MakemapWidget(qt.QWidget):
    """
    Widget for calibrating the parameters. It shows in a scatterView the tth and eta values of the data,
    to help the user perform the calibration.
    """

    def __init__(self, parent=None):
        qt.QMainWindow.__init__(self, parent)

        self.dataset = None
        self._thresholds = {}

        layout = qt.QGridLayout()
        self._parametersFilename = FilenameSelectionWidget(
            "Load parameters file", parent=self, fileMode=qt.QFileDialog.ExistingFile
        )
        self._ubiFilename = FilenameSelectionWidget(
            "Load UBI file", parent=self, fileMode=qt.QFileDialog.ExistingFile
        )
        self._newUbiFilename = FilenameSelectionWidget(
            "Load new ubi filename", parent=self, acceptMode=qt.QFileDialog.AcceptSave
        )
        self._fltFilename = FilenameSelectionWidget(
            "Load new flt filename", parent=self, acceptMode=qt.QFileDialog.AcceptSave
        )
        self._computeB = qt.QPushButton("Compute map", self)
        self._computeB.pressed.connect(self.compute)
        self._computeB.setEnabled(False)
        self._thresholdsCB = qt.QComboBox(parent=self)
        self._thresholdsCB.currentIndexChanged.connect(self._changeThreshold)
        self._loadPeaks = qt.QPushButton("Load peaks")
        self._loadPeaks.pressed.connect(self.loadPeaks)
        validator = qt.QDoubleValidator(self)
        validator.setLocale(qt.QLocale(qt.QLocale.English, qt.QLocale.UnitedStates))
        validator.setBottom(0)
        self._tolerance = qt.QLineEdit("0.05")
        self._tolerance.setValidator(validator)
        self._tth_range = qt.QLineEdit("(0, 180)")
        self._no_sort = qt.QCheckBox("No sort")
        self._omega_no_float = qt.QCheckBox("Omega no float")
        self._omega_slop = qt.QLineEdit("")
        self._omega_slop.setValidator(validator)
        self._latticeCB = qt.QComboBox(parent=self)
        self._latticeCB.addItems(Lattice.values())
        self._plot1 = PlotWindow(self)
        self._plot1.hide()
        self._plot2 = PlotWindow(self)
        self._plot2.hide()
        thresh_widget = qt.QWidget()
        thresh_layout = qt.QHBoxLayout()
        thresh_layout.addWidget(qt.QLabel("Thresholds:"))
        thresh_layout.addWidget(self._thresholdsCB)
        thresh_layout.addWidget(self._loadPeaks)
        thresh_layout.addWidget(self._tolerance)
        thresh_widget.setLayout(thresh_layout)
        layout.addWidget(thresh_widget, 0, 0, 1, 2)
        layout.addWidget(qt.QLabel("Tolerance:"), 0, 2)
        layout.addWidget(self._tolerance, 0, 3)
        layout.addWidget(qt.QLabel("Lattice:"), 0, 4)
        layout.addWidget(self._latticeCB, 0, 5)
        omega_layout = qt.QHBoxLayout()
        omega_widget = qt.QWidget()
        omega_layout.addWidget(qt.QLabel("Omega slop:"))
        omega_layout.addWidget(self._omega_slop)
        omega_widget.setLayout(omega_layout)
        layout.addWidget(omega_widget, 1, 0, 1, 2)
        layout.addWidget(self._omega_no_float, 1, 3)
        layout.addWidget(qt.QLabel("Tth range:"), 1, 2)
        layout.addWidget(self._tth_range, 1, 3)
        layout.addWidget(self._parametersFilename, 2, 0, 1, 3)
        layout.addWidget(self._ubiFilename, 2, 3, 1, 3)
        layout.addWidget(self._newUbiFilename, 3, 0, 1, 3)
        layout.addWidget(self._fltFilename, 3, 3, 1, 3)
        layout.addWidget(self._computeB, 7, 1, 1, 4)
        self._plot1.setMinimumHeight(200)
        layout.addWidget(self._plot1, 4, 0, 3, 3)
        layout.addWidget(self._plot2, 4, 3, 3, 3)
        layout.addWidget(self._no_sort, 1, 4)
        self.setLayout(layout)

    def setDataset(self, dataset):
        """
        Set dataset and fill thresholdsCB with dataset's peaks files.
        """
        self.dataset = dataset
        oldState = self._thresholdsCB.blockSignals(True)
        self._thresholdsCB.clear()
        self._thresholdsCB.blockSignals(oldState)
        self._thresholdsCB.addItems(
            [str(peaks.threshold) for peaks in self.dataset.peaks_groups]
        )
        self._loadPeaks.setEnabled(True)
        self._computeB.setEnabled(True)

        if self.dataset.peaks_groups[0].parameters is not None:
            self._parametersFilename.setFilename(
                self.dataset.peaks_groups[0].parameters
            )
        if self.dataset.peaks_groups[0].ubi_file is not None:
            self._parametersFilename.setFilename(
                self.dataset.peaks_groups[0].ubi_file
            )

    def _changeThreshold(self, index):
        """
        Peak new file with different threshold peaks.
        """
        if self.dataset is None:
            threshold = self._thresholdsCB.currentText()
            self.peaksFile = self._thresholds[threshold]
        else:
            peaks = self.dataset.peaks_groups[index]
            self.peaksFile = peaks.flt_file
            if peaks is not None:
                self._parametersFilename.setFilename(
                    peaks.parameters if peaks.parameters is not None else ""
                )
                self._ubiFilename.setFilename(
                    peaks.ubi_file if peaks.ubi_file is not None else ""
                )

    def loadPeaks(self):
        """
        Load peaks from file
        """
        fileDialog = qt.QFileDialog(self, "Load peaks")
        fileDialog.setFileMode(qt.QFileDialog.ExistingFile)
        fileDialog.setNameFilter(
            "Flt files (*.flt);;Spt files (*.spt);;All files (*);;"
        )
        if fileDialog.exec_():
            filename = fileDialog.selectedFiles()[0]
            filename_spt = filename[:-3] + "spt"
            i = filename.find("_t")
            threshold = filename[i + 2:-4]
            try:
                if self.dataset is None:
                    self._thresholds[threshold] = filename
                    self._thresholdsCB.addItem(threshold)
                    self._thresholdsCB.setCurrentText(threshold)
                else:
                    threshold = float(threshold)
                    self.dataset.add_peaks(threshold, filename, filename_spt)
                    oldState = self._thresholdsCB.blockSignals(True)
                    self._thresholdsCB.clear()
                    self._thresholdsCB.blockSignals(oldState)
                    self._thresholdsCB.addItems(
                        [str(peaks.threshold) for peaks in self.dataset.peaks_groups]
                    )
                    self._thresholdsCB.setCurrentText(str(threshold))
            except ValueError:
                print("Could not find threshold in filename")
        else:
            _logger.warning("Could not open file")

    def compute(self):
        """
        Compute refinement of grains positions and plot histograms of the before
        and the after.
        """
        assert (
            self._ubiFilename.getFilename() and self._ubiFilename.getFilename()
        ), "Parameters file and UBI file must be given"
        omega_float = self._omega_no_float.isChecked()
        omega_slop = self._omega_slop.text()
        omega_slop = float(omega_slop) if omega_slop else None
        tthr = str(self._tth_range.text())
        tth_range = (float(tthr[1]), float(tthr[-2]))

        try:
            # func = getattr(ImageD11.refinegrains, self.latticesymmetry)
            refiner = refinegrains(
                intensity_tth_range=tth_range, OmFloat=omega_float, OmSlop=omega_slop
            )
        except Exception:
            raise
            refiner = refinegrains()

        refiner.loadparameters(self._parametersFilename.getFilename())
        refiner.loadfiltered(self.peaksFile)
        refiner.readubis(self._ubiFilename.getFilename())
        symmetry = self._latticeCB.currentText()
        if symmetry != "triclinic":
            # Grainspotter will have already done this
            refiner.makeuniq(symmetry)
        refiner.tolerance = float(self._tolerance.text())
        refiner.generate_grains()
        refiner.refinepositions()
        # o.refineubis(quiet = False , scoreonly = True)
        newubi = (
            self._newUbiFilename.getFilename()
            if self._newUbiFilename.getFilename()
            else self._ubiFilename.getFilename()[:-4] + ".map"
        )
        newflt = (
            self._fltFilename.getFilename()
            if self._fltFilename.getFilename()
            else self.peaksFile[:-4] + ".new"
        )
        sort = not self._no_sort.isChecked()
        refiner.savegrains(newubi, sort_npks=sort)
        refiner.scandata[self.peaksFile].writefile(newflt)

        self.plot_grain_hist(
            self._plot1, self.peaksFile, self._ubiFilename.getFilename()
        )
        self.plot_grain_hist(self._plot2, self.peaksFile, newubi)

        self._plot1.show()
        self._plot2.show()

    def plot_grain_hist(self, plot, peaks_file, ubis_file):
        """
        Plot histogram of data corresponding to the fitted grains.

        :param plot: widget where to plot the different data.
        :param peaks_file: filename with the peaks.
        :param ubis_file: filename with the UBI matrices.
        """
        refinement = refinegrains(OmFloat=False)
        refinement.loadparameters(self._parametersFilename.getFilename())
        refinement.readubis(ubis_file)
        refinement.loadfiltered(peaks_file)
        refinement.tolerance = float(self._tolerance.text())
        refinement.generate_grains()
        refinement.assignlabels()
        data = refinement.scandata[peaks_file]
        data.filter(data.labels >= 0)
        drlv_bins = numpy.linspace(0, float(self._tolerance.text()), 15)
        ng = int(data.labels.max()) + 1
        drlv = numpy.sqrt(data.drlv2)
        dp5 = [drlv[data.labels == i] for i in range(ng)]
        hl = [numpy.histogram(dpi, drlv_bins)[0] for dpi in dp5]
        if drlv_bins.shape[0] != hl[0].shape[0]:
            plotbins = (drlv_bins[1:] + drlv_bins[:-1]) / 2
        for i in range(ng):
            plot.addCurve(plotbins, hl[i], legend=str(i))



# coding: utf-8
# /*##########################################################################
#
# Copyright (c) 2016-2017 European Synchrotron Radiation Facility
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
#
# ###########################################################################*/


__authors__ = ["J. Garriga"]
__license__ = "MIT"
__date__ = "12/08/2020"

from silx.gui import qt
from silx.gui.colors import Colormap
from silx.gui.plot.StackView import StackViewMainWindow
from .dataSelectionWidget import FilenameSelectionWidget

from darfix.gui.operationThread import OperationThread


class BackgroundSubtractionWidget(qt.QWidget):
    """
    Widget to apply background subtraction to a dataset
    """

    def __init__(self, parent=None):
        qt.QWidget.__init__(self, parent)

        self.setLayout(qt.QVBoxLayout())
        self._compute = qt.QPushButton("Compute background subtraction")
        self._compute.pressed.connect(self.__computeBS)
        widget = qt.QWidget()
        layout = qt.QHBoxLayout()
        layout.addWidget(qt.QLabel("Step:"))
        self._step = qt.QLineEdit("1")
        self._outputFilename = FilenameSelectionWidget(
            "Load output filename", parent=self, acceptMode=qt.QFileDialog.AcceptSave
        )
        self._outputFilename.setFilename("background.edf")
        layout.addWidget(self._step)
        layout.addWidget(self._outputFilename)
        widget.setLayout(layout)
        self._sv = StackViewMainWindow()
        self._sv.setColormap(Colormap("magma", normalization="log"))
        self.layout().addWidget(widget)
        self.layout().addWidget(self._sv)
        self.layout().addWidget(self._compute)

    def __computeBS(self):
        """
        Function that starts the thread to compute the background
        subtraction.
        #"""
        # if not self.dataset.in_memory:
        #     chunks = [int(self._parametersDock.verticalChunkSize.text()),
        #               int(self._parametersDock.horizontalChunkSize.text())]
        self._compute.setEnabled(False)
        outputFilename = self._outputFilename.getFilename()
        self._thread = OperationThread(self, self.dataset.compute_background)
        self._thread.setArgs(outputFilename, step=int(self._step.text()))
        self._thread.finished.connect(self._updateData)
        self._thread.start()

    def _updateData(self):
        """
        Updates the stack with the data computed in the thread
        """
        self._thread.finished.disconnect(self._updateData)
        if self._thread.data is not None:
            self.dataset.data.background = self._thread.data
            self.setStack()
        else:
            print("\nComputation aborted")
        self._compute.setEnabled(True)

    def setDataset(self, dataset):
        """
        Dataset setter. Saves the dataset and updates the stack with the dataset
        data

        :param Dataset dataset: dataset
        """
        self.dataset = dataset
        self.setStack()

    def getDataset(self):
        return self._update_dataset, self.indices, self.bg_indices, self.bg_dataset

    def resetStack(self):
        self.setStack()

    def clearStack(self):
        self._sv.setStack(None)

    def getStack(self):
        return self._sv.getStack(False, True)[0]

    def setStack(self, dataset=None):
        """
        Sets new data to the stack.
        Mantains the current frame showed in the view.

        :param Dataset dataset: if not None, data set to the stack will be from the given dataset.
        """
        if dataset is None:
            dataset = self.dataset
        nframe = self._sv.getFrameNumber()
        self._sv.clear()
        self._sv.setStack(dataset.data, reset=True)
        self._sv.setFrameNumber(nframe + 1)
        self._sv.setFrameNumber(nframe)



# coding: utf-8
# /*##########################################################################
#
# Copyright (c) 2016-2017 European Synchrotron Radiation Facility
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
#
# ###########################################################################*/


__authors__ = ["J. Garriga"]
__license__ = "MIT"
__date__ = "11/08/2020"

import logging

from silx.gui import qt
from darfix.gui.operationThread import OperationThread
from .dataSelectionWidget import FilenameSelectionWidget
from .showPeaksWidget import ShowPeaksWidget

_logger = logging.getLogger(__file__)


class PeakSearchWidget(qt.QMainWindow):

    sigComputed = qt.Signal()

    def __init__(self, parent=None):
        qt.QMainWindow.__init__(self, parent)

        self.dataset = None

        thresholdLabel = qt.QLabel("Threshold:")
        self._threshold = qt.QLineEdit("", parent=self)
        omegaLabel = qt.QLabel("Omega:")
        self._omega = qt.QLineEdit("0", parent=self)
        omegaStepLabel = qt.QLabel("Omega step:")
        self._omegaStep = qt.QLineEdit("1", parent=self)
        self._omegaOverride = qt.QCheckBox("Omega override", self)
        self._outputFilename = FilenameSelectionWidget(
            "Load output filename", parent=self, acceptMode=qt.QFileDialog.AcceptSave
        )
        self._computeButton = qt.QPushButton("Search peaks")
        self._computeButton.setEnabled(False)
        self._computeButton.pressed.connect(self.computePeakSearch)

        widget = qt.QWidget(self)
        widget.setLayout(qt.QGridLayout())
        widget.layout().addWidget(thresholdLabel, 0, 0)
        widget.layout().addWidget(self._threshold, 0, 1)
        widget.layout().addWidget(omegaLabel, 0, 2)
        widget.layout().addWidget(self._omega, 0, 3)
        widget.layout().addWidget(omegaStepLabel, 1, 0)
        widget.layout().addWidget(self._omegaStep, 1, 1)
        widget.layout().addWidget(self._omegaOverride, 1, 2, 1, 2, qt.Qt.AlignRight)
        widget.layout().addWidget(self._outputFilename, 3, 0, 1, 4)
        widget.layout().addWidget(self._computeButton, 4, 3)

        self._showPeaks = ShowPeaksWidget()
        # Main widget is a Splitter with the top widget and the displayComponentsWidget
        self.splitter = qt.QSplitter(qt.Qt.Vertical)
        self.splitter.addWidget(widget)
        self.splitter.addWidget(self._showPeaks)
        self.setCentralWidget(self.splitter)

    def setDataset(self, dataset):
        self.dataset = dataset
        self._showPeaks.setDataset(dataset)
        self._computeButton.setEnabled(True)
        self._thread = OperationThread(self, self.dataset.peak_search)

    def computePeakSearch(self):

        if not self._threshold.text():
            raise ValueError("Threshold value must be filled")
        outputFilename = self._outputFilename.getFilename()
        thresholds = [float(self._threshold.text())]
        omega = float(self._omega.text())
        omega_step = float(self._omegaStep.text())
        omega_override = self._omegaOverride.isChecked()
        self._computeButton.setEnabled(False)

        if not outputFilename:
            outputFilename = "peak_search"
        self._thread.setArgs(
            thresholds, outputFilename, omega, omega_override, omega_step
        )
        self._thread.start()
        self._thread.finished.connect(self._updateData)

    def _updateData(self):
        self._thread.finished.disconnect(self._updateData)
        self._showPeaks.addPeaks(self.dataset.peaks_groups[-1].threshold)
        self._computeButton.setEnabled(True)

        self.sigComputed.emit()



# coding: utf-8
# /*##########################################################################
#
# Copyright (c) 2016-2017 European Synchrotron Radiation Facility
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
#
# ###########################################################################*/


__authors__ = ["J. Garriga"]
__license__ = "MIT"
__date__ = "17/07/2020"

import fabio
import unittest
import numpy
import os
import tempfile

from i3dxrd.dataset import Dataset


class TestDataset(unittest.TestCase):

    """Tests for class Dataset in `dataset.py`"""

    def setUp(self):
        _dir = tempfile.mkdtemp()
        for index in range(10):
            data_file = os.path.join(_dir, "data_file%04i.edf" % index)
            data = numpy.ones((100, 100))
            data[25:50, 25:50] = numpy.random.random((25, 25))
            image = fabio.edfimage.EdfImage(data=data)
            image.write(data_file)

        self.dataset = Dataset(first_filename=_dir + "/data_file0000.edf", in_memory=True)
        self.dataset2 = Dataset(
            first_filename=_dir + "/data_file0000.edf", in_memory=False
        )

    def test_nframes(self):
        """Tests the nframes method"""
        self.assertEqual(self.dataset.nframes, 10)

    def test_bs(self):
        background1 = self.dataset.compute_background()
        background2 = self.dataset2.compute_background()

        numpy.testing.assert_array_equal(background1, background2)



# coding: utf-8
# /*##########################################################################
#
# Copyright (c) 2016-2017 European Synchrotron Radiation Facility
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
#
# ###########################################################################*/


__authors__ = ["J. Garriga"]
__license__ = "MIT"
__date__ = "02/12/2020"

import os

import fabio

import numpy

from fabio.fabioimage import fabioimage
from ImageD11 import blobcorrector
from ImageD11.correct import correct
from ImageD11.labelimage import labelimage
from ImageD11.peaksearcher import peaksearch
import silx
from silx.io.url import DataUrl
from darfix.core.dataset import Data


class Dataset:
    """Class to define a dataset from a series of data files.

    :param str first_filename: First filename of the series of files to read.
    :param int nframes: Number of frames to read.
    :param list filenames: Ordered list of filenames, defaults to None.
    :type filenames: Union[Generator,Iterator,List], optional
    :param str dark_filename: Filename for dark file.
    :param str flood_filename: Filename for flood file.
    :param str spline_filename: Filename for spline file.
    :param in_memory: If True, data is loaded into memory, else, data is read in
        chunks depending on the algorithm to apply, defaults to False.
    :type in_memory: bool, optional
    """

    def __init__(
        self,
        first_filename=None,
        nframes=None,
        filenames=None,
        dark_filename=None,
        flood_filename=None,
        spline_filename=None,
        isH5=False,
        in_memory=False,
    ):

        assert (
            first_filename is not None or filenames is not None
        ), "\
            Either first filename or filename must be given"
        self._peaks_groups = []
        self._dark_filename = dark_filename
        self._flood_filename = flood_filename
        self._spline_filename = spline_filename
        self._isH5 = isH5
        self._in_memory = in_memory

        metadata = []
        data_urls = []

        if self._isH5:
            url = DataUrl(first_filename)
            with silx.io.open(url.file_path()) as h5:
                rot = h5["entry0000/sample/rotation_angle"][:] % 360
                # rocking = h5['entry0000/sample/rocking'][:]
                # n_steps_rotation = h5['entry0000/sample/n_steps_rotation'][0]
                # n_steps_rocking = h5['entry0000/sample/n_steps_rocking']
                for i in range(h5[url.data_path()].shape[0]):
                    data_urls.append(
                        DataUrl(
                            file_path=url.file_path(),
                            data_path=url.data_path(),
                            data_slice=i,
                            scheme="silx",
                        )
                    )
                    metadata.append({"Omega": rot[i]})
        else:
            with fabio.open_series(
                filenames=filenames, first_filename=first_filename
            ) as series:
                for frame in series.frames():
                    filename = frame.file_container.filename
                    data_urls.append(DataUrl(file_path=filename, scheme="fabio"))
                    metadata.append(frame.header)

        self._data = Data_3DXRD(
            numpy.array(data_urls), metadata=metadata, in_memory=self._in_memory
        )

    @property
    def file_series(self):
        return self._file_series

    @property
    def nframes(self):
        """
        Return number of frames
        """
        if self.data is None:
            return 0
        else:
            return len(self.data.flatten())

    @property
    def peaks_groups(self):
        return self._peaks_groups

    def add_peaks(self, threshold, flt_file=None, spt_file=None):
        peaks = PeaksData(threshold)
        peaks.flt_file = flt_file
        peaks.spt_file = spt_file
        self._peaks_groups.append(peaks)
        return peaks

    @property
    def in_memory(self):
        return self._data.in_memory

    @property
    def data(self):
        return self._data

    def get_frame(self, i):
        return self._data[i]

    def compute_background(self, bg_filename="background.edf", step=1):
        """Function that computes the background from the dataset data by
        applying the median.

        :param str bg_filename: filename to use as output for the background
        :param int step: spacing between values, defaults to 1.
        :returns: ndarray
        :raises: ValueError
        """
        background = numpy.zeros(self._data[0].shape, self._data.dtype)
        if self.in_memory:
            numpy.median(self.data, out=background, axis=0)
        else:
            indices = numpy.arange(0, len(self.data), step)
            numpy.median(
                Data(self.data.urls[indices], self.data.metadata[indices]),
                out=background, axis=0)

        im = fabio.edfimage.edfimage(data=background)
        try:
            im.write(bg_filename, force_type=im.data.dtype)
        except TypeError:
            im.write(bg_filename)

        return background

    def peak_search(
        self, thresholds, out_file, OMEGA=0, OMEGAOVERRIDE=True, OMEGASTEP=1
    ):
        """
        Method to compute peak search as done in ImageD11.
        """

        li_objs = {}  # label image objects, dict of

        s = self._data[0].shape  # data array shape
        # Output files:
        if out_file[-4:] != ".spt":
            out_file = out_file + ".spt"
            print("Your output file must end with .spt, changing to ", out_file)

        if self._spline_filename is not None and os.path.exists(self._spline_filename):
            print("Using spatial from", self._spline_filename)
            corr_func = blobcorrector.correctorclass(self._spline_filename)
        else:
            print("Avoiding spatial correction")
            corr_func = blobcorrector.perfect()

        # Create label images
        for t in thresholds:
            # the last 4 chars are guaranteed to be .spt above
            mergefile = "%s_t%d.flt" % (out_file[:-4], t)
            spotfile = "%s_t%d.spt" % (out_file[:-4], t)
            li_objs[t] = labelimage(
                shape=s, fileout=mergefile, spatial=corr_func, sptfile=spotfile
            )
            print("make labelimage", mergefile, spotfile)

        for i in range(len(self._data)):
            data_object = fabioimage(self._data[i], self._data.metadata[i])
            # t = timer()
            filein = self._data.urls[i]
            if OMEGAOVERRIDE or "Omega" not in data_object.header:
                data_object.header["Omega"] = OMEGA
                OMEGA += OMEGASTEP
                OMEGAOVERRIDE = True  # once you do it once, continue
            # TODO: add median, monitorval and monitorcol
            data_object = correct(
                data_object, self._dark_filename, self._flood_filename
            )
            # t.tick(filein+" io/cor")
            peaksearch(filein, data_object, corr_func, thresholds, li_objs)
        for t in li_objs:
            self.add_peaks(t, li_objs[t].outfile.name, li_objs[t].sptfile.name)


class Data_3DXRD(Data):
    def __new__(cls, urls, metadata, in_memory=True):
        obj = super(Data_3DXRD, cls).__new__(cls, urls, metadata, in_memory)

        obj._subtract_background = False

        return obj

    def __array_finalize__(self, obj):
        if obj is None:
            return
        self.urls = getattr(obj, "urls", None)
        self.metadata = getattr(obj, "metadata", None)
        self.in_memory = getattr(obj, "in_memory", None)
        self._subtract_background = getattr(obj, "_subtract_background", None)

    @property
    def subtract_background(self):
        return self._subtract_background

    @subtract_background.setter
    def subtract_background(self, subtract_background):
        self._subtract_background = subtract_background

    @property
    def background(self):
        return self._background

    @background.setter
    def background(self, background):
        self._background = background
        self._subtract_background = True

    def __subtract_background(self, data):
        if data.dtype.kind == "i" or data.dtype.kind == "u":
            new_data = numpy.subtract(
                data, self.background.astype(data.dtype), dtype=numpy.int64
            )
        else:
            new_data = numpy.subtract(data, self.background.astype(data.dtype))
        new_data[new_data < 0] = 0

        return new_data.astype(data.dtype)


class PeaksData:
    def __init__(self, threshold):

        self._threshold = threshold
        self._flt_file = None
        self._spt_file = None
        self._parameters = None
        self._ubi_file = None
        self._gvectors = None

    @property
    def parameters(self):
        return self._parameters

    @parameters.setter
    def parameters(self, parameters):
        self._parameters = parameters

    @property
    def ubi_file(self):
        return self._ubi_file

    @ubi_file.setter
    def ubi_file(self, ubi_file):
        self._ubi_file = ubi_file

    @property
    def gvectors(self):
        return self._gvectors

    @gvectors.setter
    def gvectors(self, gvectors):
        self._gvectors = gvectors

    @property
    def threshold(self):
        return self._threshold

    @property
    def flt_file(self):
        return self._flt_file

    @flt_file.setter
    def flt_file(self, flt_file):
        self._flt_file = flt_file

    @property
    def spt_file(self):
        return self._spt_file

    @spt_file.setter
    def spt_file(self, spt_file):
        self._spt_file = spt_file



from fabio.fabioimage import fabioimage
import numpy as np
import h5py


def fso(filename, dataset):
    with h5py.File(filename, "r") as h:
        omega = h["/1.1/measurement/diffrz"][:] % 360  # forwards scan
        nframes = len(omega)
        data = h[dataset]
        assert data.shape[0] == nframes
        header = {"Omega": omega[0]}
        filename = "%s::%s" % (filename, dataset)
        order = np.argsort(omega)

        def frm(i):
            header["Omega"] = omega[i]
            f = fabioimage(data[i], header)
            f.filename = filename
            f.currentframe = i
            return f

        yield (frm(order[0]))  # first
        for i in order:
            yield frm(i)



# coding: utf-8
# /*##########################################################################
#
# Copyright (c) 2016-2017 European Synchrotron Radiation Facility
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
#
# ###########################################################################*/


__authors__ = ["J. Garriga"]
__license__ = "MIT"
__date__ = "30/03/2021"

import logging
import numpy

from silx.gui import qt
from silx.gui.colors import Colormap
from silx.gui.plot import ScatterView
from ImageD11 import transformer

_logger = logging.getLogger(__file__)


class CalibrationWidget(qt.QMainWindow):
    """
    Widget for calibrating the parameters. It shows in a scatterView the tth and eta values of the data,
    to help the user perform the calibration.
    """

    def __init__(self, parent=None):
        qt.QMainWindow.__init__(self, parent)

        self.dataset = None

        widget = qt.QWidget()
        layout = qt.QGridLayout()
        self._transformer = transformer.transformer()

        self._thresholdsCB = qt.QComboBox(parent=self)
        self._thresholdsCB.currentIndexChanged.connect(self._changeThreshold)
        self._editParameters = qt.QPushButton("Edit parameters")
        self._editParameters.pressed.connect(self.showParameters)
        self._loadParameters = qt.QPushButton("Load parameters")
        self._loadParameters.pressed.connect(self.loadParameters)
        self._addCellPeaksB = qt.QPushButton("Add cell peaks")
        self._addCellPeaksB.pressed.connect(self.addCellPeaks)
        self._saveGvB = qt.QPushButton("Save g-vectors")
        self._saveGvB.pressed.connect(self.saveGVectors)
        self._fitB = qt.QPushButton("Fit")
        self._fitB.pressed.connect(self.fit)
        self._shownCellPeaks = False

        self._scatterView = ScatterView(backend="gl")
        # import pdb; pdb.set_trace()
        if not self._scatterView._plot()._backend.isValid():
            self._scatterView = ScatterView(backend="matplotlib")
        self._scatterView.setColormap(Colormap(name="cividis", normalization="log"))
        self._scatterView.getMaskToolsWidget().parent().setFloating(True)
        self._scatterView.getMaskToolsWidget().show()
        self._scatterView.getScatterItem().setSymbolSize(1.0)

        thresh_layout = qt.QHBoxLayout()

        thresh_layout.addWidget(qt.QLabel("Thresholds:"))
        thresh_layout.addWidget(self._thresholdsCB)
        thresh_widget = qt.QWidget()
        thresh_widget.setLayout(thresh_layout)
        thresh_layout.setContentsMargins(0, 0, 0, 0)
        layout.addWidget(thresh_widget, 0, 0, 1, 1)
        layout.addWidget(self._editParameters, 0, 1)
        layout.addWidget(self._fitB, 0, 2)
        layout.addWidget(self._loadParameters, 1, 0)
        layout.addWidget(self._addCellPeaksB, 1, 1)
        layout.addWidget(self._saveGvB, 1, 2)
        layout.addWidget(self._scatterView, 2, 0, 1, 3)
        widget.setLayout(layout)
        self.setCentralWidget(widget)

    def setDataset(self, dataset):
        self.dataset = dataset
        oldState = self._thresholdsCB.blockSignals(True)
        self._thresholdsCB.clear()
        self._thresholdsCB.blockSignals(oldState)
        self._thresholdsCB.addItems(
            [str(peaks.threshold) for peaks in self.dataset.peaks_groups]
        )

    def addCellPeaks(self):
        """
        Add cell peaks to the plot.
        """
        self._shownCellPeaks = True
        self._transformer.addcellpeaks()
        theory_tth = self._transformer.theorytth
        self._scatterView.getPlotWidget().remove(kind="curve")
        curve = self._scatterView.getPlotWidget().addCurve(
            theory_tth,
            numpy.zeros(theory_tth.shape[0]),
            symbol="|",
            linestyle=" ",
            color="red",
        )
        self._scatterView.getPlotWidget().getCurve(curve).setSymbolSize(20.0)

    def fit(self):
        """
        Fit parameters with vary to make the computed two-theta positions match
        the observed ones.
        """
        tthmin, tthmax = self._scatterView.getPlotWidget().getXAxis().getLimits()
        self._transformer.fit(tthmin, tthmax)
        tth, eta = self._transformer.compute_tth_eta()
        inds = numpy.array(self._transformer.indices).flatten()
        values = numpy.zeros((len(tth),))
        for inds in self._transformer.indices:
            values[inds] = numpy.ones(len(inds)) * numpy.random.random() + 0.5
        self.plotTthEta(values)

    def saveGVectors(self):
        """
        Save g-vectors, i.e scattering vectors, into file.
        """
        self._transformer.computegv()
        fileDialog = qt.QFileDialog(self, "Save g-vectors")
        fileDialog.setFileMode(qt.QFileDialog.AnyFile)
        fileDialog.setAcceptMode(qt.QFileDialog.AcceptSave)
        fileDialog.setDefaultSuffix(".gve")
        fileDialog.setNameFilter("G-vectors files (*.gve *.gv);;All files (*);;")
        if fileDialog.exec_():
            filename = fileDialog.selectedFiles()[0]
            self._transformer.savegv(filename)
            i = self._thresholdsCB.currentIndex()
            self.dataset.peaks_groups[i].gvectors = filename
        else:
            _logger.warning("Could not open file")

    def _changeThreshold(self, index):
        """
        Pick up new file with different threshold peaks.
        """
        self._thresholdsCB.setEnabled(False)
        peaks = self.dataset.peaks_groups[index]
        try:
            self._transformer.loadfiltered(peaks.flt_file)
        except IndexError:
            _logger.warning("No peaks to plot")
        self._thresholdsCB.setEnabled(True)
        self.plotTthEta()

    def plotTthEta(self, values=None):
        """
        Plot tth and eta values.
        """
        try:
            tth, eta = self._transformer.compute_tth_eta()
            self._thresholdsCB.setEnabled(True)
            self._scatterView.setData(
                tth, eta, values if values is not None else numpy.zeros((len(tth),))
            )
            self._scatterView.resetZoom()
            if self._shownCellPeaks:
                self.addCellPeaks()
        except Exception as e:
            _logger.warning(e)

    def applyParameters(self):
        """
        Update plot with new parameters.
        """
        result = self._paramsDialog.result()
        varys = []
        pars = {}
        for key in result:
            pars[key] = result[key][0]
            if result[key][1]:
                varys.append(key)
        self._transformer.parameterobj.set_parameters(pars)
        self._transformer.parameterobj.set_varylist(varys)
        self.plotTthEta()

    def saveParameters(self, filename):
        """
        Save parameters to file.
        """
        self.applyParameters()
        self._transformer.saveparameters(filename)
        self.dataset.peaks_groups[self._thresholdsCB.currentIndex()].parameters = filename

    def showParameters(self):
        """
        Show parameters dialog.
        """
        possvars = self._transformer.get_variable_list()
        _vars = self._transformer.getvars()
        logic = {}
        for v in possvars:
            if v in _vars:
                logic[v] = 1
            else:
                logic[v] = 0
        self._paramsDialog = ParametersDialog(self, self._transformer.pars, logic)
        self._paramsDialog.applyParametersSignal.connect(self.applyParameters)
        self._paramsDialog.saveParametersSignal.connect(self.saveParameters)
        if self._paramsDialog.exec_() == qt.QDialog.Accepted:
            self.applyParameters()

    def loadParameters(self):
        """
        Load parameters from file
        """
        fileDialog = qt.QFileDialog(self, "Load parameters")
        fileDialog.setFileMode(qt.QFileDialog.ExistingFile)
        fileDialog.setNameFilter(
            "Parameters files (*.par *.prm, *.prms);;All files (*);;"
        )
        if fileDialog.exec_():
            filename = fileDialog.selectedFiles()[0]
            self._transformer.loadfileparameters(filename)
            if self.dataset is not None:
                peaks = self.dataset.peaks_groups[self._thresholdsCB.currentIndex()]
                peaks.parameters = filename
            self.plotTthEta()
        else:
            _logger.warning("Could not open file")


class ParametersDialog(qt.QDialog):
    """
    Dialog for showing and editing the parameters.
    """

    saveParametersSignal = qt.Signal(str)
    applyParametersSignal = qt.Signal()

    def __init__(self, parent=None, parameters=None, variable_list=None):
        super(ParametersDialog, self).__init__(parent=parent)

        self._transformer = transformer
        self._slider = None
        self._focusedPar = None

        self.setWindowTitle("Detector parameters")

        QBtn = (
            qt.QDialogButtonBox.Ok
            | qt.QDialogButtonBox.Cancel
            | qt.QDialogButtonBox.Save
            | qt.QDialogButtonBox.Apply
        )

        self.buttonBox = qt.QDialogButtonBox(QBtn)
        self.buttonBox.button(qt.QDialogButtonBox.Save).clicked.connect(
            self.saveParameters
        )
        self.buttonBox.button(qt.QDialogButtonBox.Apply).clicked.connect(
            self.applyParameters
        )
        self.buttonBox.button(qt.QDialogButtonBox.Ok).clicked.connect(self.accept)
        self.buttonBox.rejected.connect(self.reject)

        self.layout = qt.QVBoxLayout()

        class MyLineEdit(qt.QLineEdit):

            focusInSignal = qt.Signal()

            def focusInEvent(self, event):
                self.focusInSignal.emit()
                super(MyLineEdit, self).focusInEvent(event)

        if parameters is not None:
            self._keys = list(parameters.keys())
            self._keys.sort()
            self.pars_layout = qt.QGridLayout()
            self._parametersW = {}
            num_param = int(len(parameters) / 2)

            for i, key in enumerate(self._keys):
                if variable_list is not None:
                    par = MyLineEdit(str(parameters[key]))
                    par.focusInSignal.connect(self.parameterFocused)
                    vary = qt.QCheckBox("Vary?")
                    if i <= num_param:
                        self.pars_layout.addWidget(qt.QLabel(key), i * 2, 0)
                        self.pars_layout.addWidget(par, i * 2, 1)
                        if key in variable_list:
                            vary.setChecked(variable_list[key])
                            self.pars_layout.addWidget(vary, i * 2, 2)
                    else:
                        self.pars_layout.addWidget(
                            qt.QLabel(key), (i - num_param - 1) * 2, 3
                        )
                        self.pars_layout.addWidget(par, (i - num_param - 1) * 2, 4)
                        if key in variable_list:
                            vary.setChecked(variable_list[key])
                            self.pars_layout.addWidget(vary, (i - num_param - 1) * 2, 5)
                    self._parametersW[key] = (par, vary)
                else:
                    par = MyLineEdit(str(parameters[key]))
                    par.focusInSignal.connect(self.parameterFocused)
                    if i <= num_param:
                        self.pars_layout.addWidget(qt.QLabel(key), i * 2, 0)
                        self.pars_layout.addWidget(par, i * 2, 1)
                    else:
                        self.pars_layout.addWidget(
                            qt.QLabel(key), (i - num_param - 1) * 2, 3
                        )
                        self.pars_layout.addWidget(par, (i - num_param - 1) * 2, 4)
                    self._parametersW[key] = par

            paramWidget = qt.QWidget(parent=self)
            paramWidget.setLayout(self.pars_layout)
            self.layout.addWidget(paramWidget)
        self.layout.addWidget(self.buttonBox)
        self.setLayout(self.layout)

    def parameterFocused(self):
        if self._slider is not None:
            self.pars_layout.removeWidget(self._slider)
        pos = self.pars_layout.getItemPosition(
            self.pars_layout.indexOf(self.focusWidget())
        )
        self._focusedPar = self.focusWidget()
        self._slider = qt.QSlider(qt.Qt.Horizontal)
        self._focusedPar.textEdited.connect(self._updateSlider)
        value = self._focusedPar.text()
        try:
            self._updateSlider(value)
            self._slider.valueChanged.connect(self._updateLineEdit)
            self.pars_layout.addWidget(self._slider, pos[0] + 1, pos[1])
        except ValueError:
            pass

    def _updateLineEdit(self, new_value):
        value = self._focusedPar.text()
        decimal_number = value[::-1].find(".")
        if decimal_number == -1:
            if "e" in value and value.split("e")[1][0] == "-":
                decimal_number = int(value.split("e")[1][1:])
            else:
                decimal_number = 0
        self._focusedPar.setText(
            "{0:.{precision}f}".format(
                float(new_value) / (10**decimal_number), precision=decimal_number
            )
        )

    def _updateSlider(self, value):

        oldState = self._slider.blockSignals(True)
        try:
            decimal_number = value[::-1].find(".")
            if decimal_number == -1 and "e" in value and value.split("e")[1][0] == "-":
                decimal_number = int(value.split("e")[1][1:])
                min_r = float(value) - 100 * 10 ** (-decimal_number)
                max_r = float(value) + 100 * 10 ** (-decimal_number)
                self._slider.setRange(
                    int(min_r * 10**decimal_number), int(max_r * 10**decimal_number)
                )
                self._slider.setValue(int(float(value) * 10**decimal_number))
        except ValueError:
            _logger.warning("Slider value can't be string")

        self._slider.blockSignals(oldState)

    def saveParameters(self):
        """
        Open dialog to choose file for saving the parameters and emit signal.
        """
        fileDialog = qt.QFileDialog(self, "Save parameters")
        fileDialog.setFileMode(qt.QFileDialog.AnyFile)
        fileDialog.setAcceptMode(qt.QFileDialog.AcceptSave)
        fileDialog.setDefaultSuffix(".par")
        fileDialog.setNameFilter(
            "Parameters files (*.par *.prm, *.prms);;All files (*);;"
        )
        if fileDialog.exec_():
            filename = fileDialog.selectedFiles()[0]
            self.saveParametersSignal.emit(filename)
        else:
            _logger.warning("Could not open file")

    def applyParameters(self):
        """
        Emit signal for applying parameters.
        """
        self.applyParametersSignal.emit()

    def result(self):
        """
        Returns list with dictionary containing the modified parameters.
        """
        result = {}
        for key in self._parametersW.keys():
            result[key] = (
                (
                    self._parametersW[key][0].text(),
                    self._parametersW[key][1].isChecked(),
                )
                if isinstance(self._parametersW[key], tuple)
                else self._parametersW[key].text()
            )
        return result



# coding: utf-8
# /*##########################################################################
#
# Copyright (c) 2016-2017 European Synchrotron Radiation Facility
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
#
# ###########################################################################*/


__authors__ = ["J. Garriga"]
__license__ = "MIT"
__date__ = "28/01/2021"

import logging
import os

import numpy

from silx.gui import qt
from silx.gui.colors import Colormap
from silx.gui.plot3d.SceneWindow import SceneWindow
from silx.gui.plot3d import items
from silx.gui.plot import PlotWindow

from ImageD11 import indexing

from .calibrationWidget import ParametersDialog

_logger = logging.getLogger(__file__)


class _ParametersDock(qt.QDockWidget):
    def __init__(self, parent=None):
        """
        Dock widget containing the input parameters.
        """
        qt.QDockWidget.__init__(self, parent)
        widget = qt.QWidget()
        layout = qt.QGridLayout()

        # Init buttons
        self.thresholdsCB = qt.QComboBox(parent=self)
        thresh_layout = qt.QHBoxLayout()
        thresh_layout.addWidget(qt.QLabel("Thresholds:"))
        thresh_layout.addWidget(self.thresholdsCB)
        thresh_widget = qt.QWidget()
        thresh_widget.setLayout(thresh_layout)
        thresh_layout.setContentsMargins(0, 0, 0, 0)
        self.loadGVectors = qt.QPushButton("Load g-vectors")
        self.assignPeaks = qt.QPushButton("Assign peaks to powder rings")
        self.editParameters = qt.QPushButton("Edit parameters")
        self.loadParameters = qt.QPushButton("Load parameters")
        self.generateOrientations = qt.QPushButton("Generate trial orientations")
        self.generateOrientations.setEnabled(False)
        self.scoreOrientations = qt.QPushButton("Score trial orientations")
        self.scoreOrientations.setEnabled(False)
        layout.addWidget(thresh_widget, 0, 0, 1, 2)
        layout.addWidget(self.loadGVectors, 0, 2, 1, 2)
        layout.addWidget(self.assignPeaks, 0, 4, 1, 2)
        layout.addWidget(self.editParameters, 2, 0, 1, 3)
        layout.addWidget(self.loadParameters, 2, 3, 1, 3)
        layout.addWidget(self.generateOrientations, 3, 0, 1, 3)
        layout.addWidget(self.scoreOrientations, 3, 3, 1, 3)

        widget.setLayout(layout)
        self.setWidget(widget)


class IndexingWidget(qt.QMainWindow):
    """
    Widget for indexing the orientations of the sample to the corresponding grains.
    """

    def __init__(self, parent=None):
        qt.QMainWindow.__init__(self, parent)

        self.dataset = None

        self._thresholds = {}

        # Indexer
        self._indexer = indexing.indexer()

        self._parametersDock = _ParametersDock()
        self._parametersDock.setFeatures(qt.QDockWidget.DockWidgetMovable)
        self._parametersDock.scoreOrientations.pressed.connect(
            self.scoreTrialOrientations
        )
        self._parametersDock.loadGVectors.pressed.connect(self.loadGVectors)
        self._parametersDock.assignPeaks.pressed.connect(self.assignPeaks)
        self._parametersDock.editParameters.pressed.connect(self.showParameters)
        self._parametersDock.loadParameters.pressed.connect(self.loadParameters)
        self._parametersDock.generateOrientations.pressed.connect(
            self.generateTrialOrientations
        )
        self._parametersDock.thresholdsCB.currentIndexChanged.connect(
            self._changeThreshold
        )
        self.addDockWidget(qt.Qt.TopDockWidgetArea, self._parametersDock)

        self.bottomDock = qt.QDockWidget(self)
        self._plotHist = qt.QPushButton("Plot histograms of UBI matrices")
        self._saveUBI = qt.QPushButton("Save UBI matrices")
        self._plotHist.pressed.connect(self.plotHist)
        self._saveUBI.pressed.connect(self.saveUBI)
        self._plot = PlotWindow(self)
        self._plot.setMinimumHeight(200)
        self._plot.hide()

        layout = qt.QGridLayout()
        layout.addWidget(self._plotHist, 0, 0)
        layout.addWidget(self._saveUBI, 0, 2)
        layout.addWidget(self._plot, 1, 0, 1, 3)
        widget = qt.QWidget()
        widget.setLayout(layout)

        self.bottomDock.setWidget(widget)
        self.bottomDock.setEnabled(False)
        self.addDockWidget(qt.Qt.BottomDockWidgetArea, self.bottomDock)

        # Create a SceneWindow
        window = SceneWindow(self)

        # First scatter
        self._sceneWidget = window.getSceneWidget()
        self._sceneWidget.setBackgroundColor((0.8, 0.8, 0.8, 1.0))
        self._sceneWidget.setForegroundColor((1.0, 1.0, 1.0, 1.0))
        self._sceneWidget.setTextColor((0.1, 0.1, 0.1, 1.0))
        self._scatter1 = items.Scatter3D()
        self._scatter1.setColormap(Colormap(name="hsv"))
        self._scatter1.setSymbol("d")
        SIZE = 1024
        self._scatter1.setScale(SIZE, SIZE, SIZE)
        self._scatter1.setSymbolSize(11)  # Set the size of the markers
        self._sceneWidget.addItem(self._scatter1)

        # Second scatter
        self._scatter2 = items.Scatter3D()
        self._scatter2.setSymbol("o")
        self._scatter2.getColormap().setName("gray")  # Use 'magma' colormap
        self._scatter2.setScale(SIZE, SIZE, SIZE)
        self._scatter2.setSymbolSize(11)  # Set the size of the markers
        self._scatter2.setLabel(
            "Scatter with non-assigned peaks"
        )  # Set name displayed in parameter tree
        self.setCentralWidget(window)

    def setDataset(self, dataset):
        self.dataset = dataset
        oldState = self._parametersDock.thresholdsCB.blockSignals(True)
        self._parametersDock.thresholdsCB.clear()
        self._parametersDock.thresholdsCB.blockSignals(oldState)
        self._parametersDock.thresholdsCB.addItems(
            [str(peaks.threshold) for peaks in self.dataset.peaks_groups]
        )

    def _changeThreshold(self, index):
        """
        Pick up new file with different threshold peaks.
        """
        self._parametersDock.thresholdsCB.setEnabled(False)
        filename = (
            self.dataset.peaks_groups[index].gvectors
            if self.dataset is not None
            else self._thresholds[self._parametersDock.thresholdsCB.currentText()]
        )

        try:
            self._indexer.readgvfile(filename)
            self._scatter1.setLabel(
                "Scatter with gvectors"
            )  # Set name displayed in parameter tree
            self.plotXYZ(self._indexer.gv.T)
        except (IndexError, TypeError):
            _logger.warning("No gvectors to plot")
        self._parametersDock.thresholdsCB.setEnabled(True)

    def loadGVectors(self):
        """
        Load g-vectors from file
        """
        fileDialog = qt.QFileDialog(self, "Load g-vectors")
        fileDialog.setFileMode(qt.QFileDialog.ExistingFile)
        fileDialog.setNameFilter("Parameters files (*.gv *.gve);;All files (*);;")
        if fileDialog.exec_():
            filename = fileDialog.selectedFiles()[0]
            try:
                i = filename.find("_t")
                threshold = filename[i + 2:-4]
            except ValueError:
                threshold = os.basename(filename)
                _logger.warning("Could not find threshold in filename")
            if self.dataset is not None:
                oldState = self._parametersDock.thresholdsCB.blockSignals(True)
                peaks = self.dataset.add_peaks(threshold)
                peaks.gvectors = filename
                self._parametersDock.thresholdsCB.clear()
                self._parametersDock.thresholdsCB.blockSignals(oldState)
                self._parametersDock.thresholdsCB.addItem(threshold)
                self._parametersDock.thresholdsCB.setCurrentText(str(threshold))
            else:
                self._thresholds[threshold] = filename
                self._parametersDock.thresholdsCB.addItem(threshold)
                self._parametersDock.thresholdsCB.setCurrentText(str(threshold))
            self._indexer.readgvfile(filename)
            self.plotXYZ(self._indexer.gv.T)
        else:
            _logger.warning("Could not open file")

    def generateTrialOrientations(self):
        """
        Find orientations.
        """
        self._indexer.find()
        self._parametersDock.scoreOrientations.setEnabled(True)

    def scoreTrialOrientations(self):
        """
        Score orientations
        """
        self._indexer.scorethem()

    def saveUBI(self):
        """
        Save UBI matrices into file.
        """
        fileDialog = qt.QFileDialog(self, "Save UBI")
        fileDialog.setFileMode(qt.QFileDialog.AnyFile)
        fileDialog.setAcceptMode(qt.QFileDialog.AcceptSave)
        fileDialog.setNameFilter("UBI files (*.ubi);;All files (*);;")
        if fileDialog.exec_():
            filename = fileDialog.selectedFiles()[0]
            self._indexer.saveubis(filename)
            idx = self._parametersDock.thresholdsCB.currentIndex()
            self.dataset.peaks_groups[idx].ubi_file = filename
        else:
            _logger.warning("Could not open file")

    def plotHist(self):
        """
        Plot histogram of the difference between calculated orientations and
        errors in peak position.
        """
        self._indexer.histogram_drlv_fit()
        x = self._indexer.histogram
        y = self._indexer.bins
        self._plot.clear()
        for xline in x:
            self._plot.addHistogram(xline, y[1:], legend=str(xline))
        self._plot.show()

    def plotXYZ(self, peaks1, peaks2=None, values1=None, values2=None):
        """
        Plot scatter with corresponding data.
        """
        self._scatter1.setData(
            peaks1[0],
            peaks1[1],
            peaks1[2],
            numpy.zeros(peaks1.shape[1]) if values1 is None else values1,
        )
        if peaks2 is not None:
            self._scatter2.setData(
                peaks2[0],
                peaks2[1],
                peaks2[2],
                numpy.zeros(peaks2.shape[1]) if values2 is None else values2,
            )
        else:
            if self._scatter2 in self._sceneWidget.getItems():
                self._sceneWidget.removeItem(self._scatter2)
        self._parametersDock.generateOrientations.setEnabled(True)
        self.bottomDock.setEnabled(True)

    def assignPeaks(self):
        """
        Assign peaks to powder rings
        """
        if self._scatter2 not in self._sceneWidget.getItems():
            self._sceneWidget.addItem(self._scatter2)
        self._scatter1.setLabel(
            "Scatter with assigned peaks"
        )  # Set name displayed in parameter tree
        self._indexer.assigntorings()
        N = len(numpy.unique(self._indexer.ra))
        values1 = (self._indexer.ra[self._indexer.ra != -1] * (N + 1) // 2) % N
        values2 = self._indexer.ra[self._indexer.ra == -1]
        peaks1 = self._indexer.gv[self._indexer.ra != -1].T
        peaks2 = self._indexer.gv[self._indexer.ra == -1].T
        self.plotXYZ(peaks1, peaks2, values1, values2)

    def applyParameters(self):
        """
        Update plot with new parameters.
        """
        result = self._paramsDialog.result()
        self._indexer.parameterobj.set_parameters(result)
        self._indexer.loadpars()

    def saveParameters(self, filename):
        """
        Save parameters to file.
        """
        result = self._paramsDialog.result()
        self._indexer.parameterobj.set_parameters(result)
        self._indexer.saveparameters(filename)

    def showParameters(self):
        """
        Show parameters dialog.
        """
        self._indexer.updateparameters()
        self._paramsDialog = ParametersDialog(self, self._indexer.pars)
        self._paramsDialog.applyParametersSignal.connect(self.applyParameters)
        self._paramsDialog.saveParametersSignal.connect(self.saveParameters)
        if self._paramsDialog.exec_() == qt.QDialog.Accepted:
            self.applyParameters()

    def loadParameters(self):
        """
        Load parameters from file
        """
        fileDialog = qt.QFileDialog(self, "Load parameters")
        fileDialog.setFileMode(qt.QFileDialog.ExistingFile)
        fileDialog.setNameFilter(
            "Parameters files (*.par *.prm, *.prms);;All files (*);;"
        )
        if fileDialog.exec_():
            filename = fileDialog.selectedFiles()[0]
            self._indexer.loadpars(filename)
        else:
            _logger.warning("Could not open file")



# coding: utf-8
# /*##########################################################################
#
# Copyright (c) 2016-2017 European Synchrotron Radiation Facility
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
#
# ###########################################################################*/


__authors__ = ["J. Garriga"]
__license__ = "MIT"
__date__ = "22/07/2020"

import logging

from silx.gui import qt
from silx.gui.colors import Colormap
from silx.gui.plot import ScatterView

from ImageD11.columnfile import columnfile

_logger = logging.getLogger(__file__)


class ShowPeaksWidget(qt.QMainWindow):
    def __init__(self, parent=None):
        qt.QMainWindow.__init__(self, parent)

        colWidget = self._initColPlot()
        self.setCentralWidget(colWidget)

    def _initColPlot(self):

        widget = qt.QWidget()
        layout = qt.QGridLayout()

        self._thresholdsCB = qt.QComboBox(parent=self)
        self._thresholdsCB.currentIndexChanged.connect(self._changeThreshold)
        bxa = qt.QComboBox(None)
        bya = qt.QComboBox(None)
        bza = qt.QComboBox(None)
        self._axisboxes = [bxa, bya, bza]
        self._ignoreselect = True
        for box in self._axisboxes:
            box.currentIndexChanged.connect(self.select)
        self._loadPeaks = qt.QPushButton("Load peaks")
        self._loadPeaks.pressed.connect(self.loadPeaks)

        self._scatterView = ScatterView(backend="gl")
        # import pdb; pdb.set_trace()
        if not self._scatterView._plot()._backend.isValid():
            self._scatterView = ScatterView(backend="matplotlib")
        self._scatterView.setColormap(Colormap(name="cividis", normalization="log"))
        self._scatterView.getMaskToolsWidget().parent().setFloating(True)
        self._scatterView.getMaskToolsWidget().show()

        layout.addWidget(qt.QLabel("Thresholds:"), 0, 0)
        layout.addWidget(self._thresholdsCB, 0, 1)
        layout.addWidget(self._loadPeaks, 0, 2)
        layout.addWidget(self._scatterView, 1, 0, 1, 3)
        layout.addWidget(qt.QLabel("x-plot"), 2, 0)
        layout.addWidget(qt.QLabel("y-plot"), 2, 1)
        layout.addWidget(qt.QLabel("color"), 2, 2)
        layout.addWidget(bxa, 3, 0)
        layout.addWidget(bya, 3, 1)
        layout.addWidget(bza, 3, 2)
        widget.setLayout(layout)

        return widget

    def select(self, col):
        """
        Choose the x,y,z axes for plotting
        """
        if self._ignoreselect:
            return
        self.update()
        self._scatterView.resetZoom()

    def setDataset(self, dataset):
        self.dataset = dataset
        oldState = self._thresholdsCB.blockSignals(True)
        self._thresholdsCB.clear()
        self._thresholdsCB.blockSignals(oldState)
        self._thresholdsCB.addItems(
            [str(peaks.threshold) for peaks in self.dataset.peaks_groups]
        )

    def addPeaks(self, threshold):
        self._thresholdsCB.addItem(str(threshold))

    def loadPeaks(self):
        """
        Load peaks from file
        """
        fileDialog = qt.QFileDialog(self, "Load peaks")
        fileDialog.setFileMode(qt.QFileDialog.ExistingFile)
        fileDialog.setNameFilter(
            "Flt files (*.flt);;Spt files (*.spt);;All files (*);;"
        )
        if fileDialog.exec_():
            filename = fileDialog.selectedFiles()[0]
            filename_spt = filename[:-3] + "spt"
            i = filename.find("_t")
            threshold = filename[i + 2:-4]
            try:
                threshold = float(threshold)
                self.dataset.add_peaks(threshold, filename, filename_spt)
                oldState = self._thresholdsCB.blockSignals(True)
                self._thresholdsCB.clear()
                self._thresholdsCB.blockSignals(oldState)
                self._thresholdsCB.addItems(
                    [str(peaks.threshold) for peaks in self.dataset.peaks_groups]
                )
                self._thresholdsCB.setCurrentText(str(threshold))
            except ValueError:
                _logger.warning("Could not find threshold in filename")
        else:
            _logger.warning("Could not open file")

    def _changeThreshold(self, idx):
        self._thresholdsCB.setEnabled(False)
        filename = self.dataset.peaks_groups[idx].flt_file
        self.openColFile(filename)
        self._thresholdsCB.setEnabled(True)

    def openColFile(self, filename):

        try:
            self._colfile = columnfile(filename)
        except OSError:
            print("problem opening file", filename)
            return
        except IndexError:
            print("Problem reading file (maybe empty?)")
            return

        self.setTitles()
        self.update()
        self._scatterView.resetZoom()

    def setTitles(self):
        """Read the colfile titles into the dropdowns"""
        self._ignoreselect = True
        for i, b in enumerate(self._axisboxes):
            t = b.currentText()
            b.clear()
            b.addItems(self._colfile.titles)
            if t in self._colfile.titles:
                b.setCurrentIndex(self._colfile.titles.index(t))
            else:
                b.setCurrentIndex(i)
        self._ignoreselect = False

    def update(self):
        """Refreshes the plot"""
        self.x = self._colfile.getcolumn(self._axisboxes[0].currentText())
        self.y = self._colfile.getcolumn(self._axisboxes[1].currentText())
        self.z = self._colfile.getcolumn(self._axisboxes[2].currentText())
        self._scatterView.getXAxis().setLabel(self._axisboxes[0].currentText())
        self._scatterView.getYAxis().setLabel(self._axisboxes[1].currentText())
        self._scatterView.setGraphTitle("color: " + self._axisboxes[2].currentText())
        self._scatterView.setData(self.x, self.y, self.z)



# coding: utf-8
# /*##########################################################################
#
# Copyright (c) 2016-2017 European Synchrotron Radiation Facility
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
#
# ###########################################################################*/


__authors__ = ["Jérôme Kieffer", "Thomas Vincent", "Júlia Garriga"]
__license__ = "MIT"
__date__ = "29/01/2021"

import io
import logging
import os
import sys

logging.basicConfig(level=logging.INFO)

logger = logging.getLogger("i3dxrd.setup")

from setuptools import find_packages

try:
    from setuptools import Command

    logger.info("Use setuptools")
except ImportError:
    try:
        from numpy.distutils.core import Command
    except ImportError:
        from distutils.core import Command
    logger.info("Use distutils")

try:
    import sphinx
    import sphinx.util.console

    sphinx.util.console.color_terminal = lambda: False
    from sphinx.setup_command import BuildDoc
except ImportError:
    sphinx = None

with open("README.rst", "r") as fh:
    long_description = fh.read()

PROJECT = "i3dxrd"


def get_version():
    # Returns current version number from version.py file
    dirname = os.path.dirname(os.path.abspath(__file__))
    sys.path.insert(0, dirname)
    import i3dxrd.version

    sys.path = sys.path[1:]
    return i3dxrd.version.strictversion


classifiers = [
    "Development Status :: 3 - Alpha",
    "Environment :: Console",
    "Environment :: X11 Applications :: Qt",
    "Intended Audience :: Education",
    "Intended Audience :: Science/Research",
    "License :: OSI Approved :: MIT License",
    "Natural Language :: English",
    "Operating System :: POSIX",
    "Programming Language :: Python :: 3.4",
    "Programming Language :: Python :: 3.5",
    "Programming Language :: Python :: 3.6",
    "Programming Language :: Python :: 3.7",
]


def get_readme():
    """Returns content of README.rst file"""
    dirname = os.path.dirname(os.path.abspath(__file__))
    filename = os.path.join(dirname, "README.rst")
    with io.open(filename, "r", encoding="utf-8") as fp:
        long_description = fp.read()
    return long_description


# ############################# #
# numpy.distutils Configuration #
# ############################# #


def configuration(parent_package="", top_path=None):
    """Recursive construction of package info to be used in setup().

    See http://docs.scipy.org/doc/numpy/reference/distutils.html#numpy.distutils.misc_util.Configuration
    """
    try:
        from numpy.distutils.misc_util import Configuration
    except ImportError:
        raise ImportError(
            "To install this package, you must install numpy first\n"
            "(See https://pypi.python.org/pypi/numpy)"
        )
    config = Configuration(None, parent_package, top_path)
    config.set_options(
        ignore_setup_xxx_py=True,
        assume_default_configuration=True,
        delegate_options_to_subpackages=True,
        quiet=True,
    )
    config.add_subpackage(PROJECT)
    return config


# ################### #
# build_doc command   #
# ################### #

if sphinx is None:

    class SphinxExpectedCommand(Command):
        """Command to inform that sphinx is missing"""

        user_options = []

        def initialize_options(self):
            pass

        def finalize_options(self):
            pass

        def run(self):
            raise RuntimeError(
                "Sphinx is required to build or test the documentation.\n"
                "Please install Sphinx (http://www.sphinx-doc.org)."
            )


class BuildMan(Command):
    """Command to build man pages"""

    user_options = []

    def initialize_options(self):
        pass

    def finalize_options(self):
        pass

    def entry_points_iterator(self):
        """Iterate other entry points available on the project."""
        entry_points = self.distribution.entry_points
        console_scripts = entry_points.get("console_scripts", [])
        gui_scripts = entry_points.get("gui_scripts", [])
        scripts = []
        scripts.extend(console_scripts)
        scripts.extend(gui_scripts)
        for script in scripts:
            # Remove ending extra dependencies
            script = script.split("[")[0]
            elements = script.split("=")
            target_name = elements[0].strip()
            elements = elements[1].split(":")
            module_name = elements[0].strip()
            function_name = elements[1].strip()
            yield target_name, module_name, function_name

    def run_targeted_script(self, target_name, script_name, env, log_output=False):
        """Execute targeted script using --help and --version to help checking
        errors. help2man is not very helpful to do it for us.
        :return: True is both return code are equal to 0
        :rtype: bool
        """
        import subprocess

        if log_output:
            extra_args = {}
        else:
            try:
                # Python 3
                from subprocess import DEVNULL
            except ImportError:
                # Python 2
                import os

                DEVNULL = open(os.devnull, "wb")
            extra_args = {"stdout": DEVNULL, "stderr": DEVNULL}

        succeeded = True
        command_line = [sys.executable, script_name, "--help"]
        if log_output:
            logger.info("See the following execution of: %s", " ".join(command_line))
        p = subprocess.Popen(command_line, env=env, **extra_args)
        status = p.wait()
        if log_output:
            logger.info("Return code: %s", status)
        succeeded = succeeded and status == 0
        command_line = [sys.executable, script_name, "--version"]
        if log_output:
            logger.info("See the following execution of: %s", " ".join(command_line))
        p = subprocess.Popen(command_line, env=env, **extra_args)
        status = p.wait()
        if log_output:
            logger.info("Return code: %s", status)
        succeeded = succeeded and status == 0
        return succeeded

    def run(self):
        build = self.get_finalized_command("build")
        path = sys.path
        path.insert(0, os.path.abspath(build.build_lib))

        env = dict((str(k), str(v)) for k, v in os.environ.items())
        env["PYTHONPATH"] = os.pathsep.join(path)

        import subprocess

        status = subprocess.call(["mkdir", "-p", "build/man"])
        if status != 0:
            raise RuntimeError("Fail to create build/man directory")

        import tempfile
        import stat

        script_name = None

        entry_points = self.entry_points_iterator()
        for target_name, module_name, function_name in entry_points:
            logger.info("Build man for entry-point target '%s'" % target_name)

            # help2man expect a single executable file to extract the help
            # we create it, execute it, and delete it at the end
            py3 = sys.version_info >= (3, 0)
            try:
                # create a launcher using the right python interpreter
                script_fid, script_name = tempfile.mkstemp(
                    prefix="%s_" % target_name, text=True
                )
                script = os.fdopen(script_fid, "wt")
                script.write("#!%s\n" % sys.executable)
                script.write("import %s as app\n" % module_name)
                script.write("app.%s()\n" % function_name)
                script.close()
                # make it executable
                mode = os.stat(script_name).st_mode
                os.chmod(script_name, mode + stat.S_IEXEC)

                # execute help2man
                man_file = "build/man/%s.1" % target_name
                command_line = ["help2man", script_name, "-o", man_file]
                if not py3:
                    # Before Python 3.4, ArgParser --version was using
                    # stderr to print the version
                    command_line.append("--no-discard-stderr")
                    # Then we dont know if the documentation will contains
                    # durtty things
                    succeeded = self.run_targeted_script(
                        target_name, script_name, env, False
                    )
                    if not succeeded:
                        logger.info(
                            "Error while generating man file for target '%s'.",
                            target_name,
                        )
                        self.run_targeted_script(target_name, script_name, env, True)
                        raise RuntimeError(
                            "Fail to generate '%s' man documentation" % target_name
                        )

                p = subprocess.Popen(command_line, env=env)
                status = p.wait()
                if status != 0:
                    logger.info(
                        "Error while generating man file for target '%s'.", target_name
                    )
                    self.run_targeted_script(target_name, script_name, env, True)
                raise RuntimeError(
                    "Fail to generate '%s' man documentation" % target_name
                )
            finally:
                # clean up the script
                if script_name is not None:
                    os.remove(script_name)


if sphinx is not None:

    class BuildDocCommand(BuildDoc):
        """Command to build documentation using sphinx.

        Project should have already be built.
        """

        def run(self):
            # make sure the python path is pointing to the newly built
            # code so that the documentation is built on this and not a
            # previously installed version

            build = self.get_finalized_command("build")
            sys.path.insert(0, os.path.abspath(build.build_lib))

            # Build the Users Guide in HTML and TeX format
            for builder in ["html", "latex"]:
                self.builder = builder
                self.builder_target_dir = os.path.join(self.build_dir, builder)
                self.mkpath(self.builder_target_dir)
                BuildDoc.run(self)
            sys.path.pop(0)

else:
    BuildDocCommand = SphinxExpectedCommand


# ################### #
# test_doc command    #
# ################### #

if sphinx is not None:

    class TestDocCommand(BuildDoc):
        """Command to test the documentation using sphynx doctest.

        http://www.sphinx-doc.org/en/1.4.8/ext/doctest.html
        """

        def run(self):
            # make sure the python path is pointing to the newly built
            # code so that the documentation is built on this and not a
            # previously installed version

            build = self.get_finalized_command("build")
            sys.path.insert(0, os.path.abspath(build.build_lib))

            # Build the Users Guide in HTML and TeX format
            for builder in ["doctest"]:
                self.builder = builder
                self.builder_target_dir = os.path.join(self.build_dir, builder)
                self.mkpath(self.builder_target_dir)
                BuildDoc.run(self)
            sys.path.pop(0)

else:
    TestDocCommand = SphinxExpectedCommand


# ##### #
# setup #
# ##### #

NAMESPACE_PACKAGES = ["orangecontrib"]

PACKAGES = find_packages()


def get_project_configuration(dry_run):
    """Returns project arguments for setup"""
    install_requires = [
        # for most of the computation
        "numpy",
        # for the script launcher
        "setuptools",
        "darfix",
        "silx",
        "ImageD11",
    ]

    doc_requires = [
        "sphinx",
        "nbsphinx",
        "recommonmark"
    ]

    full_requires = [
        "matplotlib>=1.2.0",
        "opencv-python==4.1.2.30",
        "scikit-image>=0.17.1",
        "PyQt5",
        "orange3",
    ]

    extras_require = {
        "full": full_requires,
        'doc': doc_requires
    }

    setup_requires = ["setuptools", "numpy"]

    cmdclass = dict(
        build_doc=BuildDocCommand, test_doc=TestDocCommand, build_man=BuildMan
    )

    package_data = {
        "orangecontrib.i3dxrd": [
            "widgets/icons/*.png",
            "widgets/icons/*.svg",
        ]
    }

    entry_points = {
        # Entry points that marks this package as an orange add-on. If set, addon will
        # be shown in the add-ons manager even if not published on PyPi.
        "orange3.addon": ("i3dxrd-add-on=orangecontrib.i3dxrd",),
        # Entry point used to specify packages containing widgets.
        "orange.widgets": (
            # Syntax: category name = path.to.package.containing.widgets
            # Widget category specification can be seen in
            "i3dxrd=orangecontrib.i3dxrd.widgets",
        ),
        # Register widget help
        "orange.canvas.help": (
            "html-index=orangecontrib.i3dxrd.widgets:WIDGET_HELP_PATH",
        ),
    }

    if dry_run:
        # DRY_RUN implies actions which do not require NumPy
        #
        # And they are required to succeed without Numpy for example when
        # pip is used to install tomwer when Numpy is not yet present in
        # the system.
        setup_kwargs = {}
    else:
        config = configuration()
        setup_kwargs = config.todict()
    setup_kwargs.update(
        name=PROJECT,
        version=get_version(),
        url="https://gitlab.esrf.fr/XRD/i3dxrd",
        author="data analysis unit",
        author_email="julia.garriga@esrf.fr",
        classifiers=classifiers,
        description="Computer vision software for the interpretation of diffraction images",
        long_description_content_type="text/markdown",
        long_description=get_readme(),
        namespace_packages=NAMESPACE_PACKAGES,
        packages=PACKAGES,
        install_requires=install_requires,
        extras_require=extras_require,
        setup_requires=setup_requires,
        cmdclass=cmdclass,
        package_data=package_data,
        zip_safe=False,
        entry_points=entry_points,
        license="MIT License",
        platforms="linux",
    )
    return setup_kwargs


def setup_package():
    """Run setup(**kwargs)

    Depending on the command, it either runs the complete setup which depends on numpy,
    or a *dry run* setup with no dependency on numpy.
    """

    # Check if action requires build/install
    dry_run = len(sys.argv) == 1 or (
        len(sys.argv) >= 2
        and (
            "--help" in sys.argv[1:]
            or sys.argv[1]
            in ("--help-commands", "egg_info", "--version", "clean", "--name")
        )
    )

    if dry_run:
        # DRY_RUN implies actions which do not require dependancies, like NumPy
        try:
            from setuptools import setup

            logger.info("Use setuptools.setup")
        except ImportError:
            from distutils.core import setup

            logger.info("Use distutils.core.setup")
    else:
        try:
            from setuptools import setup
        except ImportError:
            from numpy.distutils.core import setup

            logger.info("Use numpydistutils.setup")

    setup_kwargs = get_project_configuration(dry_run)
    setup(**setup_kwargs)


DATA_FILES = [
    # Data files that will be installed outside site-packages folder
]


def include_documentation(local_dir, install_dir):
    global DATA_FILES
    if "bdist_wheel" in sys.argv and not os.path.exists(local_dir):
        print(
            "Directory '{}' does not exist. "
            "Please build documentation before running bdist_wheel.".format(
                os.path.abspath(local_dir)
            )
        )
        sys.exit(0)

    doc_files = []
    for dirpath, dirs, files in os.walk(local_dir):
        doc_files.append(
            (
                dirpath.replace(local_dir, install_dir),
                [os.path.join(dirpath, f) for f in files],
            )
        )
    DATA_FILES.extend(doc_files)


if __name__ == "__main__":
    include_documentation("build/sphinx/html", "help/i3dxrd")
    setup_package()



# coding: utf-8
# /*##########################################################################
#
# Copyright (c) 2016-2017 European Synchrotron Radiation Facility
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
#
# ###########################################################################*/


__authors__ = ["J. Garriga"]
__license__ = "MIT"
__date__ = "26/07/2019"


from silx.gui import qt


class OperationThread(qt.QThread):
    """
    Given a function and a set of arguments, it calls it whenever the thread
    is started.
    """

    def __init__(self, parent, function):
        qt.QThread.__init__(self, parent=parent)
        self.func = function
        self.args = []
        self.kwargs = {}

    def setArgs(self, *args, **kwargs):
        """
        Function to set the arguments of the function

        :param List args:
        """
        self.args = args
        self.kwargs = kwargs

    def run(self):
        self.data = self.func(*self.args, **self.kwargs)



# coding: utf-8
# /*##########################################################################
#
# Copyright (c) 2016-2017 European Synchrotron Radiation Facility
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
#
# ###########################################################################*/


__authors__ = ["J. Garriga"]
__license__ = "MIT"
__date__ = "30/07/2020"

from Orange.widgets.widget import OWWidget, Input, Output

from silx.gui import qt

from i3dxrd.dataset import Dataset
from i3dxrd.gui.peakSearchWidget import PeakSearchWidget


class PeakSearchWidgetOW(OWWidget):
    """
    Widget to select the data to be used in the dataset.
    """

    name = "peak search"
    # icon = "icons/upload.svg"
    want_main_area = False

    # Inputs
    class Inputs:
        dataset = Input("dataset", Dataset)

    # Outputs
    class Outputs:
        dataset = Output("dataset", Dataset)

    def __init__(self):
        super().__init__()

        self._widget = PeakSearchWidget()
        types = qt.QDialogButtonBox.Ok
        _buttons = qt.QDialogButtonBox(parent=self)
        _buttons.setStandardButtons(types)

        self.controlArea.layout().addWidget(self._widget)
        self.controlArea.layout().addWidget(_buttons)

        _buttons.accepted.connect(self.close)

        self.__updatingData = False

    @Inputs.dataset
    def setDataset(self, dataset):
        self.show()
        if dataset:
            self._widget.setDataset(dataset)
        else:
            # Emit None
            self.Outputs.dataset.send(dataset)

    def closeEvent(self, event):
        self.Outputs.dataset.send(self._widget.dataset)
        event.accept()



# coding: utf-8
# /*##########################################################################
#
# Copyright (c) 2016-2017 European Synchrotron Radiation Facility
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
#
# ###########################################################################*/


__authors__ = ["J. Garriga"]
__license__ = "MIT"
__date__ = "14/08/2020"

import logging
import numpy

from silx.gui import qt
from ImageD11.grain import read_grain_file
from silx.gui.plot3d.SceneWindow import SceneWindow
from silx.gui.plot3d import items
from .dataSelectionWidget import FilenameSelectionWidget

_logger = logging.getLogger(__file__)


class PlotMapWidget(qt.QMainWindow):
    """
    Widget for calibrating the parameters. It shows in a scatterView the tth and eta values of the data,
    to help the user perform the calibration.
    """

    def __init__(self, parent=None):
        qt.QMainWindow.__init__(self, parent)

        widget = qt.QWidget()
        layout = qt.QVBoxLayout()
        self._grainFile = FilenameSelectionWidget(
            "Open grain file", parent=self, fileMode=qt.QFileDialog.ExistingFile
        )
        self._compute = qt.QPushButton("Compute", parent=self)
        self._compute.pressed.connect(self._loadGrains)
        layout.addWidget(self._grainFile)
        layout.addWidget(self._compute)

        # Create a SceneWindow
        self._window = SceneWindow(self)

        sceneWidget = self._window.getSceneWidget()
        sceneWidget.setBackgroundColor((0.8, 0.8, 0.8, 1.0))
        sceneWidget.setForegroundColor((1.0, 1.0, 1.0, 1.0))
        sceneWidget.setTextColor((0.1, 0.1, 0.1, 1.0))
        self._scatter3D = items.Scatter3D()
        self._scatter3D.setSymbol("x")
        self._scatter3D.getColormap().setName("temperature")  # Use 'magma' colormap
        SIZE = 1024
        self._scatter3D.setScale(SIZE, SIZE, SIZE)
        self._scatter3D.setSymbolSize(11)  # Set the size of the markers
        sceneWidget.addItem(self._scatter3D)
        self._window.hide()
        layout.addWidget(self._window)
        widget.setLayout(layout)
        self.setCentralWidget(widget)

    def _loadGrains(self):
        """
        Load grains from file
        """
        grain_file = read_grain_file(self._grainFile.getFilename())

        data = []
        intensity = []

        for g in grain_file:
            o = g.translation
            try:
                sf = pow(numpy.median(g.intensity_info), 0.3333)
            except Exception:
                sf = 1.0
            try:
                k = int(g.npks)
            except Exception:
                k = 1
            for u in g.ubi:
                data.append(o)
                data.append(o + u * sf)
                intensity.append(k)
                intensity.append(k)
            for u in g.ubi:
                data.append(o)
                data.append(o - u * sf)
                intensity.append(k)
                intensity.append(k)

        data = numpy.array(data).T
        self._scatter3D.setData(data[0], data[1], data[2], intensity)
        self._window.show()



# coding: utf-8
# /*##########################################################################
#
# Copyright (c) 2016-2017 European Synchrotron Radiation Facility
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
#
# ###########################################################################*/


__authors__ = ["J. Garriga"]
__license__ = "MIT"
__date__ = "30/11/2020"

# import os

import logging

from silx.gui import qt
from silx.gui.colors import Colormap
from silx.gui.plot.StackView import StackViewMainWindow
from ImageD11.peakmerge import peakmerger


_logger = logging.getLogger(__file__)


class StackViewWidget(qt.QWidget):
    """
    Widget that allows the user to pick a ROI in any image of the dataset.
    """

    def __init__(self, parent=None):
        qt.QWidget.__init__(self, parent)

        self.peakMerger = peakmerger()

        self.setLayout(qt.QVBoxLayout())
        self._thresholdsCB = qt.QComboBox(parent=self)
        self._thresholdsCB.currentTextChanged.connect(self._changeThreshold)
        self._loadPeaks = qt.QPushButton("Load peaks")
        self._loadPeaks.pressed.connect(self.loadPeaks)
        widget = qt.QWidget()
        layout = qt.QHBoxLayout()
        layout.addWidget(qt.QLabel("Thresholds:"))
        layout.addWidget(self._thresholdsCB)
        layout.addWidget(self._loadPeaks)
        widget.setLayout(layout)
        self._sv = StackViewMainWindow()
        self._sv.setColormap(Colormap("magma", normalization="log"))
        self.layout().addWidget(widget)
        self.layout().addWidget(self._sv)

    def setDataset(self, dataset):
        """
        Dataset setter. Saves the dataset and updates the stack with the dataset
        data

        :param Dataset dataset: dataset
        """
        self.dataset = dataset
        oldState = self._thresholdsCB.blockSignals(True)
        self._thresholdsCB.clear()
        self._thresholdsCB.blockSignals(oldState)
        self._thresholdsCB.addItems(
            [str(peaks.threshold) for peaks in self.dataset.peaks_groups]
        )
        self.setStack(dataset)

    def _changeThreshold(self, text):
        self._thresholdsCB.setEnabled(False)
        filename = self.dataset.thresholds_spt[float(text)]
        self._reloadPeaks(filename)
        self._thresholdsCB.setEnabled(True)

    def _reloadPeaks(self, filename):
        self.peakMerger.readpeaks(filename)
        self.select_image(0)
        self._sv.sigFrameChanged.connect(self.select_image)

    def loadPeaks(self):
        """
        Load peaks from file
        """
        fileDialog = qt.QFileDialog(self, "Load peaks")
        fileDialog.setFileMode(qt.QFileDialog.ExistingFile)
        fileDialog.setNameFilter(
            "Flt files (*.flt);;Spt files (*.spt);;All files (*);;"
        )
        if fileDialog.exec_():
            filename = fileDialog.selectedFiles()[0]
            filename_spt = filename[:-3] + "spt"
            i = filename.find("_t")
            threshold = filename[i + 2:-4]
            try:
                threshold = float(threshold)
                self.dataset.add_peaks(threshold, filename, filename_spt)
                oldState = self._thresholdsCB.blockSignals(True)
                self._thresholdsCB.clear()
                self._thresholdsCB.blockSignals(oldState)
                self._thresholdsCB.addItems(
                    [str(key) for key in self.dataset.thresholds_spt.keys()]
                )
                self._thresholdsCB.setCurrentText(str(threshold))
            except ValueError:
                print("Could not find threshold in filename")
        else:
            _logger.warning("Could not open file")

    def select_image(self, i):
        if i < 0 or i > len(self.peakMerger.images) - 1:
            return False
        im = self.peakMerger.images[i]
        self.currentnum = i
        j = im.imagenumber
        self.peakMerger.harvestpeaks(numlim=(j - 0.1, j + 0.1))
        self.pkx = [p.x for p in self.peakMerger.allpeaks]
        self.pky = [p.y for p in self.peakMerger.allpeaks]
        self.pkI = [p.avg for p in self.peakMerger.allpeaks]
        self.plotPeaks()
        return True

    def plotPeaks(self):

        plot = self._sv.getPlot()
        self.plotLabel = plot.addScatter(
            self.pky, self.pkx, self.pkI, z=1, symbol="+", colormap=Colormap("viridis")
        )
        plot.getScatter(self.plotLabel).setSymbolSize(10)
        plot.setKeepDataAspectRatio(True)

    def setStack(self, dataset=None):
        """
        Sets new data to the stack.
        Mantains the current frame showed in the view.

        :param Dataset dataset: if not None, data set to the stack will be from the given dataset.
        """
        if dataset is None:
            dataset = self.dataset
        nframe = self._sv.getFrameNumber()
        self._sv.setStack(dataset.data)
        self._sv.setFrameNumber(nframe)
